#ifndef __CANDEVICEDATA_H
#define __CANDEVICEDATA_H

#include <nodes/NodeDataModel>

#include <QCanBusFrame>

using QtNodes::NodeDataType;

/**
*   @brief The enum class describing frame direction
*/
enum class Direction { RX, TX, Uninitialized };

/**
*   @brief The class describing data model used as output for CanDevice node
*/
class CanDeviceDataOut : public NodeData {
public:
    CanDeviceDataOut(){};
    CanDeviceDataOut(QCanBusFrame const& frame, Direction const direction = Direction::TX, bool status = true)
        : _frame(frame)
        , _direction(direction)
        , _status(status)
    {
    }
    /**
    *   @brief  Used to get data type id and displayed text for ports
    *   @return NodeDataType of rawview
    */
    NodeDataType type() const override
    {
        return NodeDataType{ "rawframe", "RAW" };
    }

    /**
    *   @brief  Used to get frame
    */
    QCanBusFrame frame() const
    {
        return _frame;
    };

    /**
    *   @brief  Used to get direction
    */
    Direction direction() const
    {
        return _direction;
    };

    /**
    *   @brief  Used to get status
    */
    bool status() const
    {
        return _status;
    };

private:
    QCanBusFrame _frame;
    Direction _direction;
    bool _status; // used only for frameSent, ignored for frameReceived
};

using CanDeviceDataIn = CanDeviceDataOut;

#endif /* !__CANDEVICEDATA_H */
